function dict = add_2char_word(dict, word, key, A, b, n)
% dict = add_2char_word(dict, word, A, b, n)
% This function adds dictionary entries of a 2-character word until the
% n-th (or the last) position in the dictionary.
% It is used after LZW_cipher_CPA2 to make the coverage of the dictionary
% more complete to reveal more strings.
%
% Shujun Li @ www.hooklee.com 2010-2011

if nargin<2
    if ~exist('dict','var')
        dict = [];
    end
    disp('At least two input arguments are needed!');
    return;
end

if ~exist('A','var')
    plaintext_class = 'char';
    A = char([9 10 13 32:126]); % = '\t', '\n', '\r' and all other printable ASCII characters
else
    if ischar(A)
        plaintext_class = 'char';
        A = double(A); % Convert it to integers for easier handling.
    elseif (isnumeric(A) && isequal(A,fix(A)))
        plaintext_class = 'int';
        A = double(A); % Convert it to integers for easier handling.
    elseif isfloat(A)
        plaintext_class = 'float';
    else
        disp('The alphabet should be an array of chars or numbers!');
        return;
    end
end
if ~exist('b','var')
    b = 12;
end
if ~exist('n','var')
    n = 1000;
end

if n>size(dict,2)
    n = size(dict,2);
end

word = [word(1) word(2)];

AA = setdiff(A, word);
A_n = numel(AA);
Ss_n = A_n*(A_n-1)+2;
Ss = zeros(1,Ss_n);
index = 1;
for j=1:A_n
    Ss(index:index+1) = AA(1);
    index = index + 2;
    if numel(AA)>=3
        AA_n_2 = 2*(numel(AA)-2) - 1;
        Ss(index:2:index+AA_n_2) = AA(3:end);
        Ss(index+1:2:index+AA_n_2) = AA(1);
        index = index + AA_n_2 + 1;
    end
    AA(1) = []; % Always remove the first element.
end

if n>=3
    S = [word word];
    if strcmpi(plaintext_class,'char')
        disp(['Processing the first plaintext: "' S '"...']);
    else
        disp(['Processing the first plaintext: ' mat2str(S) '...']);
    end
    B = LZW_encipher(S, key, A, b, 0);
    if word(1)==word(2) % 'aaaa'
        dict{B(2)+1,2} = word;
    else % 'abab'
        dict{B(3)+1,3} = word;
    end
end

for i=1:n-3
    S = [word Ss(1:i) word];
    if strcmpi(plaintext_class,'char')
        disp(['Processing the ' iptnum2ordinal(i) ' plaintext: "' S '"...']);
    else
        disp(['Processing the ' iptnum2ordinal(i) ' plaintext: ' mat2str(S) '...']);
    end
    B = LZW_encipher(S, key, A, b, 0);
    dict{B(3+i)+1,3+i} = word;
end
