function [dict, index] = add_dict_entry(dict, string, key, b)
% [dict, index] = add_dict_entry(dict, string, key, b)
% This function adds a new dictionary entry and the 0-based index of the
% new entry.
% If the dictionary is full, index = -1.
%
% Shujun Li @ www.hooklee.com 2010-2011

index = -1;

if nargin<4
    if ~exist('dict','var')
        dict = [];
    end
    disp([mfilename ': Four input arguments are needed!']);
    return;
end

% We do not check if the dictionary is full because this may slow down this
% function. In the LZW encipher and decipher, the number of valid entries
% is counted to check if the dictionary is full.

hash_bits = num2bit(HMAC(string, key), 8);
temp = bit2num(hash_bits, b); % Convert the bits to a number of b-bit integers.
index = temp(1) + 1; % Only take the first b-bit integer.
if isempty(dict{index})
    dict{index} = string;
    index = index - 1;
else
    dict_free_flags = cellfun(@isempty, dict);
    if sum(dict_free_flags)==0
        return; % The dictionary is full.
    end
    indices = find(dict_free_flags); % Find free indices.
    if numel(indices)==1
        index = indices;
    else
        indices_n_log2 = log2(numel(indices));
        if indices_n_log2==fix(indices_n_log2) % numel(indices) is a power of 2.
            temp = bit2num(hash_bits, indices_n_log2);
            index = indices(temp(1)+1); % Take one free index.
        else
            temp = bit2num(hash_bits, 32);
            index = indices(mod(temp(1),numel(indices))+1); % Take one free index.
        end
    end
    if isempty(dict{index})
        dict{index} = string;
        index = index - 1;
    end
end
