function dict = permute_dict(dict, varargin)
% dict = permute_dict(dict, c12_r12)
% dict = permute_dict(dict, c1, c2, r1, r2)
% This function permutes the dictionary given the four parameters c1, c2,
% r1 and r2.
% The second parameter "c12_r12" should be a 4-tuple vector containing the
% four parameters.
%
% Shujun Li @ www.hooklee.com 2010-2011

if nargin<2
    if ~exist('dict','var')
        dict = [];
    end
    disp([mfilename ': At least two input arguments are needed!']);
    return;
elseif nargin<5
    c12_r12 = varargin{1};
    if (isnumeric(c12_r12) && numel(c12_r12)>=4)
        c1 = double(c12_r12(1));
        c2 = double(c12_r12(2));
        r1 = double(c12_r12(3));
        r2 = double(c12_r12(4));
    else
        disp([mfilename ': The second input argument is invalid!']);
        return;
    end
else % nargin>=5
    c1 = double(varargin{1}(1));
    c2 = double(varargin{2}(1));
    r1 = double(varargin{3}(1));
    r2 = double(varargin{4}(1));
end

M = size(dict,1);
M_1 = M - 1;

% First column permutation (actually vertical shift of odd columns).
dict(mod((0:M_1)+c1,M)+1,1:2:end) = dict(:,1:2:end);
% Second column permutation (actually vertical shift of even columns).
dict(mod((0:M_1)+c2,M)+1,2:2:end) = dict(:,2:2:end);
% First row permutation (actually horizontal shift of odd rows).
dict(1:2:end,mod((0:M_1)+r1,M)+1) = dict(1:2:end,:);
% Second row permutation (actually horizontal shift of even rows).
dict(2:2:end,mod((0:M_1)+r2,M)+1) = dict(2:2:end,:);
